/*
 * Copyright (C) 2017 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package android.car.vms;

import android.annotation.NonNull;
import android.annotation.SystemApi;
import android.os.Parcel;
import android.os.Parcelable;
import android.util.ArraySet;

import com.android.internal.util.DataClass;

import java.util.*;

/**
 * A Vehicle Map Service layer with a list of publisher IDs it is associated with.
 *
 * @hide
 */
@SystemApi
@DataClass(genAidl = true, genEqualsHashCode = true, genToString = true)
public final class VmsAssociatedLayer implements Parcelable {
    /**
     * Layer being offered
     */
    private final @NonNull VmsLayer mVmsLayer;

    /**
     * IDs of providers that publish the layer
     */
    private @NonNull Set<Integer> mProviderIds;

    private void onConstructed() {
        mProviderIds = Collections.unmodifiableSet(mProviderIds);
    }

    private void parcelProviderIds(Parcel dest, int flags) {
        dest.writeArraySet(new ArraySet<>(mProviderIds));
    }

    @SuppressWarnings("unchecked")
    private Set<Integer> unparcelProviderIds(Parcel in) {
        return (Set<Integer>) in.readArraySet(Integer.class.getClassLoader());
    }

    /**
     * IDs of publishers that publish the layer
     *
     * @deprecated Use {@link #getProviderIds()} instead
     */
    @Deprecated
    @NonNull
    public Set<Integer> getPublisherIds() {
        return mProviderIds;
    }



    // Code below generated by codegen v1.0.14.
    //
    // DO NOT MODIFY!
    // CHECKSTYLE:OFF Generated code
    //
    // To regenerate run:
    // $ codegen $ANDROID_BUILD_TOP/packages/services/Car/car-lib/src/android/car/vms/VmsAssociatedLayer.java
    //
    // To exclude the generated code from IntelliJ auto-formatting enable (one-time):
    //   Settings > Editor > Code Style > Formatter Control
    //@formatter:off


    /**
     * Creates a new VmsAssociatedLayer.
     *
     * @param vmsLayer
     *   Layer being offered
     * @param providerIds
     *   IDs of providers that publish the layer
     */
    @DataClass.Generated.Member
    public VmsAssociatedLayer(
            @NonNull VmsLayer vmsLayer,
            @NonNull Set<Integer> providerIds) {
        this.mVmsLayer = vmsLayer;
        com.android.internal.util.AnnotationValidations.validate(
                NonNull.class, null, mVmsLayer);
        this.mProviderIds = providerIds;
        com.android.internal.util.AnnotationValidations.validate(
                NonNull.class, null, mProviderIds);

        onConstructed();
    }

    /**
     * Layer being offered
     */
    @DataClass.Generated.Member
    public @NonNull VmsLayer getVmsLayer() {
        return mVmsLayer;
    }

    /**
     * IDs of providers that publish the layer
     */
    @DataClass.Generated.Member
    public @NonNull Set<Integer> getProviderIds() {
        return mProviderIds;
    }

    @Override
    @DataClass.Generated.Member
    public String toString() {
        // You can override field toString logic by defining methods like:
        // String fieldNameToString() { ... }

        return "VmsAssociatedLayer { " +
                "vmsLayer = " + mVmsLayer + ", " +
                "providerIds = " + mProviderIds +
        " }";
    }

    @Override
    @DataClass.Generated.Member
    public boolean equals(@android.annotation.Nullable Object o) {
        // You can override field equality logic by defining either of the methods like:
        // boolean fieldNameEquals(VmsAssociatedLayer other) { ... }
        // boolean fieldNameEquals(FieldType otherValue) { ... }

        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        @SuppressWarnings("unchecked")
        VmsAssociatedLayer that = (VmsAssociatedLayer) o;
        //noinspection PointlessBooleanExpression
        return true
                && Objects.equals(mVmsLayer, that.mVmsLayer)
                && Objects.equals(mProviderIds, that.mProviderIds);
    }

    @Override
    @DataClass.Generated.Member
    public int hashCode() {
        // You can override field hashCode logic by defining methods like:
        // int fieldNameHashCode() { ... }

        int _hash = 1;
        _hash = 31 * _hash + Objects.hashCode(mVmsLayer);
        _hash = 31 * _hash + Objects.hashCode(mProviderIds);
        return _hash;
    }

    @Override
    @DataClass.Generated.Member
    public void writeToParcel(@NonNull Parcel dest, int flags) {
        // You can override field parcelling by defining methods like:
        // void parcelFieldName(Parcel dest, int flags) { ... }

        dest.writeTypedObject(mVmsLayer, flags);
        parcelProviderIds(dest, flags);
    }

    @Override
    @DataClass.Generated.Member
    public int describeContents() { return 0; }

    /** @hide */
    @SuppressWarnings({"unchecked", "RedundantCast"})
    @DataClass.Generated.Member
    /* package-private */ VmsAssociatedLayer(@NonNull Parcel in) {
        // You can override field unparcelling by defining methods like:
        // static FieldType unparcelFieldName(Parcel in) { ... }

        VmsLayer vmsLayer = (VmsLayer) in.readTypedObject(VmsLayer.CREATOR);
        Set<Integer> providerIds = unparcelProviderIds(in);

        this.mVmsLayer = vmsLayer;
        com.android.internal.util.AnnotationValidations.validate(
                NonNull.class, null, mVmsLayer);
        this.mProviderIds = providerIds;
        com.android.internal.util.AnnotationValidations.validate(
                NonNull.class, null, mProviderIds);

        onConstructed();
    }

    @DataClass.Generated.Member
    public static final @NonNull Parcelable.Creator<VmsAssociatedLayer> CREATOR
            = new Parcelable.Creator<VmsAssociatedLayer>() {
        @Override
        public VmsAssociatedLayer[] newArray(int size) {
            return new VmsAssociatedLayer[size];
        }

        @Override
        public VmsAssociatedLayer createFromParcel(@NonNull Parcel in) {
            return new VmsAssociatedLayer(in);
        }
    };

    @DataClass.Generated(
            time = 1582065953302L,
            codegenVersion = "1.0.14",
            sourceFile = "packages/services/Car/car-lib/src/android/car/vms/VmsAssociatedLayer.java",
            inputSignatures = "private final @android.annotation.NonNull android.car.vms.VmsLayer mVmsLayer\nprivate @android.annotation.NonNull android.car.vms.Set<java.lang.Integer> mProviderIds\nprivate  void onConstructed()\nprivate  void parcelProviderIds(android.os.Parcel,int)\nprivate @java.lang.SuppressWarnings(\"unchecked\") android.car.vms.Set<java.lang.Integer> unparcelProviderIds(android.os.Parcel)\npublic @java.lang.Deprecated @android.annotation.NonNull android.car.vms.Set<java.lang.Integer> getPublisherIds()\nclass VmsAssociatedLayer extends java.lang.Object implements [android.os.Parcelable]\n@com.android.internal.util.DataClass(genAidl=true, genEqualsHashCode=true, genToString=true)")
    @Deprecated
    private void __metadata() {}


    //@formatter:on
    // End of generated code

}
